parent
577ca20128
commit
2c2d3f28ad
BIN
lprnet/1.jpg
Normal file
BIN
lprnet/1.jpg
Normal file
Binary file not shown.
|
After Width: | Height: | Size: 2.1 KiB |
35
lprnet/CMakeLists.txt
Normal file
35
lprnet/CMakeLists.txt
Normal file
@ -0,0 +1,35 @@
|
||||
cmake_minimum_required(VERSION 2.6)
|
||||
|
||||
project(LPRnet)
|
||||
|
||||
add_definitions(-std=c++11)
|
||||
|
||||
option(CUDA_USE_STATIC_CUDA_RUNTIME OFF)
|
||||
set(CMAKE_CXX_STANDARD 11)
|
||||
set(CMAKE_BUILD_TYPE Debug)
|
||||
|
||||
find_package(CUDA REQUIRED)
|
||||
|
||||
include_directories(${PROJECT_SOURCE_DIR}/include)
|
||||
if (CMAKE_SYSTEM_PROCESSOR MATCHES "aarch64")
|
||||
message("embed_platform on")
|
||||
include_directories(/usr/local/cuda/targets/aarch64-linux/include)
|
||||
link_directories(/usr/local/cuda/targets/aarch64-linux/lib)
|
||||
else()
|
||||
message("embed_platform off")
|
||||
include_directories(/usr/local/cuda/include)
|
||||
link_directories(/usr/local/cuda/lib64)
|
||||
# tensorrt
|
||||
include_directories(/usr/local/TensorRT-7.0.0.11/include)
|
||||
link_directories(/usr/local/TensorRT-7.0.0.11/lib)
|
||||
endif()
|
||||
|
||||
find_package(OpenCV)
|
||||
include_directories(OpenCV_INCLUDE_DIRS)
|
||||
|
||||
add_executable(LPRnet ${PROJECT_SOURCE_DIR}/LPRnet.cpp)
|
||||
target_link_libraries(LPRnet nvinfer)
|
||||
target_link_libraries(LPRnet cudart)
|
||||
target_link_libraries(LPRnet ${OpenCV_LIBS})
|
||||
|
||||
add_definitions(-O2 -pthread)
|
||||
450
lprnet/LPRnet.cpp
Normal file
450
lprnet/LPRnet.cpp
Normal file
@ -0,0 +1,450 @@
|
||||
#include <iostream>
|
||||
#include <chrono>
|
||||
#include <map>
|
||||
#include <opencv2/opencv.hpp>
|
||||
#include "NvInfer.h"
|
||||
#include "cuda_runtime_api.h"
|
||||
#include "logging.h"
|
||||
#include <fstream>
|
||||
#include <map>
|
||||
#include <sstream>
|
||||
|
||||
#define CHECK(status) \
|
||||
do\
|
||||
{\
|
||||
auto ret = (status);\
|
||||
if (ret != 0)\
|
||||
{\
|
||||
std::cerr << "Cuda failure: " << ret << std::endl;\
|
||||
abort();\
|
||||
}\
|
||||
} while (0)
|
||||
|
||||
//#define USE_FP16 // comment out this if want to use FP32
|
||||
#define DEVICE 0 // GPU id
|
||||
#define BATCH_SIZE 1
|
||||
|
||||
// stuff we know about the network and the input/output blobs
|
||||
static const int INPUT_H = 24;
|
||||
static const int INPUT_W = 94;
|
||||
static const int OUTPUT_SIZE = 18 * 68;
|
||||
const char *INPUT_BLOB_NAME = "data";
|
||||
const char *OUTPUT_BLOB_NAME = "prob";
|
||||
static Logger gLogger;
|
||||
using namespace nvinfer1;
|
||||
const std::string alphabet[] = {"京", "沪", "津", "渝", "冀", "晋", "蒙", "辽", "吉", "黑",
|
||||
"苏", "浙", "皖", "闽", "赣", "鲁", "豫", "鄂", "湘", "粤",
|
||||
"桂", "琼", "川", "贵", "云", "藏", "陕", "甘", "青", "宁",
|
||||
"新",
|
||||
"0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
|
||||
"A", "B", "C", "D", "E", "F", "G", "H", "J", "K",
|
||||
"L", "M", "N", "P", "Q", "R", "S", "T", "U", "V",
|
||||
"W", "X", "Y", "Z", "I", "O", "-"
|
||||
};
|
||||
|
||||
|
||||
// TensorRT weight files have a simple space delimited format:
|
||||
// [type] [size] <data x size in hex>
|
||||
std::map<std::string, Weights> loadWeights(const std::string file) {
|
||||
std::cout << "Loading weights: " << file << std::endl;
|
||||
std::map<std::string, Weights> weightMap;
|
||||
|
||||
// Open weights file
|
||||
std::ifstream input(file);
|
||||
assert(input.is_open() && "Unable to load weight file. please check if the .wts file path is right!!!!!!");
|
||||
|
||||
// Read number of weight blobs
|
||||
int32_t count;
|
||||
input >> count;
|
||||
assert(count > 0 && "Invalid weight map file.");
|
||||
|
||||
while (count--) {
|
||||
Weights wt{DataType::kFLOAT, nullptr, 0};
|
||||
uint32_t size;
|
||||
|
||||
// Read name and type of blob
|
||||
std::string name;
|
||||
input >> name >> std::dec >> size;
|
||||
wt.type = DataType::kFLOAT;
|
||||
|
||||
// Load blob
|
||||
uint32_t *val = reinterpret_cast<uint32_t *>(malloc(sizeof(val) * size));
|
||||
for (uint32_t x = 0, y = size; x < y; ++x) {
|
||||
input >> std::hex >> val[x];
|
||||
}
|
||||
wt.values = val;
|
||||
|
||||
wt.count = size;
|
||||
weightMap[name] = wt;
|
||||
}
|
||||
|
||||
return weightMap;
|
||||
}
|
||||
|
||||
IScaleLayer *addBatchNorm2d(INetworkDefinition *network, std::map<std::string, Weights> &weightMap, ITensor &input,
|
||||
std::string lname, float eps) {
|
||||
float *gamma = (float *) weightMap[lname + ".weight"].values;
|
||||
float *beta = (float *) weightMap[lname + ".bias"].values;
|
||||
float *mean = (float *) weightMap[lname + ".running_mean"].values;
|
||||
float *var = (float *) weightMap[lname + ".running_var"].values;
|
||||
int len = weightMap[lname + ".running_var"].count;
|
||||
|
||||
float *scval = reinterpret_cast<float *>(malloc(sizeof(float) * len));
|
||||
for (int i = 0; i < len; i++) {
|
||||
scval[i] = gamma[i] / sqrt(var[i] + eps);
|
||||
}
|
||||
Weights scale{DataType::kFLOAT, scval, len};
|
||||
|
||||
float *shval = reinterpret_cast<float *>(malloc(sizeof(float) * len));
|
||||
for (int i = 0; i < len; i++) {
|
||||
shval[i] = beta[i] - mean[i] * gamma[i] / sqrt(var[i] + eps);
|
||||
}
|
||||
Weights shift{DataType::kFLOAT, shval, len};
|
||||
|
||||
float *pval = reinterpret_cast<float *>(malloc(sizeof(float) * len));
|
||||
for (int i = 0; i < len; i++) {
|
||||
pval[i] = 1.0;
|
||||
}
|
||||
Weights power{DataType::kFLOAT, pval, len};
|
||||
|
||||
weightMap[lname + ".scale"] = scale;
|
||||
weightMap[lname + ".shift"] = shift;
|
||||
weightMap[lname + ".power"] = power;
|
||||
IScaleLayer *scale_1 = network->addScale(input, ScaleMode::kCHANNEL, shift, scale, power);
|
||||
assert(scale_1);
|
||||
return scale_1;
|
||||
}
|
||||
|
||||
IConvolutionLayer *
|
||||
small_basic_block(INetworkDefinition *network, std::map<std::string, Weights> &weightMap, ITensor &input,
|
||||
int nbOutputMaps, std::string lname) {
|
||||
IConvolutionLayer *conv = network->addConvolutionNd(input, nbOutputMaps / 4, DimsHW{1, 1},
|
||||
weightMap[lname + ".block.0.weight"],
|
||||
weightMap[lname + ".block.0.bias"]);
|
||||
|
||||
auto relu = network->addActivation(*conv->getOutput(0), ActivationType::kRELU);
|
||||
IConvolutionLayer *conv2 = network->addConvolutionNd(*relu->getOutput(0), nbOutputMaps / 4, DimsHW{3, 1},
|
||||
weightMap[lname + ".block.2.weight"],
|
||||
weightMap[lname + ".block.2.bias"]);
|
||||
conv2->setPaddingNd(DimsHW{1, 0});
|
||||
auto relu2 = network->addActivation(*conv2->getOutput(0), ActivationType::kRELU);
|
||||
|
||||
IConvolutionLayer *conv3 = network->addConvolutionNd(*relu2->getOutput(0), nbOutputMaps / 4, DimsHW{1, 3},
|
||||
weightMap[lname + ".block.4.weight"],
|
||||
weightMap[lname + ".block.4.bias"]);
|
||||
conv3->setPaddingNd(DimsHW{0, 1});
|
||||
auto relu3 = network->addActivation(*conv3->getOutput(0), ActivationType::kRELU);
|
||||
IConvolutionLayer *conv4 = network->addConvolutionNd(*relu3->getOutput(0), nbOutputMaps, DimsHW{1, 1},
|
||||
weightMap[lname + ".block.6.weight"],
|
||||
weightMap[lname + ".block.6.bias"]);
|
||||
|
||||
return conv4;
|
||||
}
|
||||
|
||||
|
||||
|
||||
ICudaEngine *createEngine(unsigned int maxBatchSize, IBuilder *builder, IBuilderConfig *config, DataType dt) {
|
||||
INetworkDefinition *network = builder->createNetworkV2(0U);
|
||||
|
||||
// Create input tensor of shape {C, INPUT_H, INPUT_W} with name INPUT_BLOB_NAME
|
||||
ITensor *data = network->addInput(INPUT_BLOB_NAME, dt, Dims4{1, 3, INPUT_H, INPUT_W});
|
||||
assert(data);
|
||||
std::map<std::string, Weights> weightMap = loadWeights("../LPRNet.wts");
|
||||
//LPRnet
|
||||
IConvolutionLayer *conv = network->addConvolutionNd(*data, 64, DimsHW{3, 3}, weightMap["backbone.0.weight"],weightMap["backbone.0.bias"]);
|
||||
assert(conv);
|
||||
ILayer *tmp = addBatchNorm2d(network, weightMap, *conv->getOutput(0), "backbone.1", 1e-5);
|
||||
auto relu = network->addActivation(*tmp->getOutput(0), ActivationType::kRELU);
|
||||
|
||||
|
||||
//f0
|
||||
auto f0 = network->addPoolingNd(*relu->getOutput(0), PoolingType::kAVERAGE, DimsHW{5, 5});
|
||||
f0->setStrideNd(DimsHW{5, 5});
|
||||
|
||||
auto p = network->addPoolingNd(*relu->getOutput(0), PoolingType::kMAX, Dims3{1, 3, 3});
|
||||
p->setStrideNd(Dims3{1, 1, 1});
|
||||
|
||||
auto small = small_basic_block(network, weightMap, *p->getOutput(0), 128, "backbone.4");
|
||||
|
||||
ILayer *tmp2 = addBatchNorm2d(network, weightMap, *small->getOutput(0), "backbone.5", 1e-5);
|
||||
auto relu2 = network->addActivation(*tmp2->getOutput(0), ActivationType::kRELU);
|
||||
|
||||
|
||||
auto f1 = network->addPoolingNd(*relu2->getOutput(0), PoolingType::kAVERAGE, DimsHW{5, 5});
|
||||
f1->setStrideNd(DimsHW{5, 5});
|
||||
|
||||
|
||||
auto p2 = network->addPoolingNd(*relu2->getOutput(0), PoolingType::kMAX, Dims3{1, 3, 3});
|
||||
p2->setStrideNd(Dims3{2, 1, 2});
|
||||
|
||||
auto small2 = small_basic_block(network, weightMap, *p2->getOutput(0), 256, "backbone.8");
|
||||
|
||||
ILayer *tmp3 = addBatchNorm2d(network, weightMap, *small2->getOutput(0), "backbone.9", 1e-5);
|
||||
auto relu3 = network->addActivation(*tmp3->getOutput(0), ActivationType::kRELU);
|
||||
|
||||
auto small3 = small_basic_block(network, weightMap, *relu3->getOutput(0), 256, "backbone.11");
|
||||
ILayer *tmp4 = addBatchNorm2d(network, weightMap, *small3->getOutput(0), "backbone.12", 1e-5);
|
||||
auto relu4 = network->addActivation(*tmp4->getOutput(0), ActivationType::kRELU);
|
||||
|
||||
auto f2 = network->addPoolingNd(*relu4->getOutput(0), PoolingType::kAVERAGE, DimsHW{4, 10});
|
||||
f2->setStrideNd(DimsHW{4, 2});
|
||||
|
||||
auto p3 = network->addPoolingNd(*relu4->getOutput(0), PoolingType::kMAX, Dims3{1, 3, 3});
|
||||
p3->setStrideNd(Dims3{4, 1, 2});
|
||||
Dims pf3 = p3->getOutput(0)->getDimensions();
|
||||
IConvolutionLayer *conv2 = network->addConvolutionNd(*p3->getOutput(0), 256, DimsHW{1, 4},
|
||||
weightMap["backbone.16.weight"],
|
||||
weightMap["backbone.16.bias"]);
|
||||
ILayer *tmp5 = addBatchNorm2d(network, weightMap, *conv2->getOutput(0), "backbone.17", 1e-5);
|
||||
auto relu5 = network->addActivation(*tmp5->getOutput(0), ActivationType::kRELU);
|
||||
|
||||
IConvolutionLayer *conv3 = network->addConvolutionNd(*relu5->getOutput(0), 68, DimsHW{13, 1},
|
||||
weightMap["backbone.20.weight"],
|
||||
weightMap["backbone.20.bias"]);
|
||||
|
||||
ILayer *tmp6 = addBatchNorm2d(network, weightMap, *conv3->getOutput(0), "backbone.21", 1e-5);
|
||||
auto backbone = network->addActivation(*tmp6->getOutput(0), ActivationType::kRELU);
|
||||
|
||||
float *deval = reinterpret_cast<float *>(malloc(sizeof(float) * 64 * 4 * 18));
|
||||
for (int i = 0; i < 64 * 4 * 18; i++) {
|
||||
deval[i] = 2.0;
|
||||
}
|
||||
Weights deconvwts11{DataType::kFLOAT, deval, 64 * 4 * 18};
|
||||
IConstantLayer *d = network->addConstant(Dims4{1, 64, 4, 18}, deconvwts11);
|
||||
IElementWiseLayer *f_pow = network->addElementWise(*f0->getOutput(0), *d->getOutput(0), ElementWiseOperation::kPOW);
|
||||
Dims pf0 = f0->getOutput(0)->getDimensions();
|
||||
Dims pD = d->getOutput(0)->getDimensions();
|
||||
Dims pf_pow = f_pow->getOutput(0)->getDimensions();
|
||||
|
||||
auto f_mean = network->addReduce(*f_pow->getOutput(0), ReduceOperation::kAVG, 0XF, true);
|
||||
Dims pf_mean = f_mean->getOutput(0)->getDimensions();
|
||||
IElementWiseLayer *f_div = network->addElementWise(*f0->getOutput(0), *f_mean->getOutput(0),
|
||||
ElementWiseOperation::kDIV);
|
||||
|
||||
Dims pf_div = f_div->getOutput(0)->getDimensions();
|
||||
float *deval2 = reinterpret_cast<float *>(malloc(sizeof(float) * 1 * 128 * 4 * 18));
|
||||
for (int i = 0; i < 128 * 4 * 18 * 1; i++) {
|
||||
deval2[i] = 2.0;
|
||||
}
|
||||
Weights deconvwts22{DataType::kFLOAT, deval2, 128 * 4 * 18 * 1};
|
||||
IConstantLayer *d2 = network->addConstant(Dims4{1, 128, 4, 18}, deconvwts22);
|
||||
IElementWiseLayer *f_pow2 = network->addElementWise(*f1->getOutput(0), *d2->getOutput(0),
|
||||
ElementWiseOperation::kPOW);
|
||||
auto f_mean2 = network->addReduce(*f_pow2->getOutput(0), ReduceOperation::kAVG, 0XF, true);
|
||||
IElementWiseLayer *f_div2 = network->addElementWise(*f1->getOutput(0), *f_mean2->getOutput(0),
|
||||
ElementWiseOperation::kDIV);
|
||||
|
||||
|
||||
float *deval3 = reinterpret_cast<float *>(malloc(sizeof(float) * 256 * 4 * 18 * 1));
|
||||
for (int i = 0; i < 256 * 4 * 18 * 1; i++) {
|
||||
deval3[i] = 2.0;
|
||||
}
|
||||
Weights deconvwts33{DataType::kFLOAT, deval3, 256 * 4 * 18 * 1};
|
||||
IConstantLayer *d3 = network->addConstant(Dims4{1, 256, 4, 18}, deconvwts33);
|
||||
IElementWiseLayer *f_pow3 = network->addElementWise(*f2->getOutput(0), *d3->getOutput(0),
|
||||
ElementWiseOperation::kPOW);
|
||||
auto f_mean3 = network->addReduce(*f_pow3->getOutput(0), ReduceOperation::kAVG, 0XF, true);
|
||||
IElementWiseLayer *f_div3 = network->addElementWise(*f2->getOutput(0), *f_mean3->getOutput(0),
|
||||
ElementWiseOperation::kDIV);
|
||||
|
||||
|
||||
float *deval4 = reinterpret_cast<float *>(malloc(sizeof(float) * 68 * 4 * 18 * 1));
|
||||
for (int i = 0; i < 68 * 4 * 18 * 1; i++) {
|
||||
deval4[i] = 2.0;
|
||||
}
|
||||
Weights deconvwts44{DataType::kFLOAT, deval4, 68 * 4 * 18 * 1};
|
||||
IConstantLayer *d4 = network->addConstant(Dims4{1, 68, 4, 18}, deconvwts44);
|
||||
IElementWiseLayer *f_pow4 = network->addElementWise(*backbone->getOutput(0), *d4->getOutput(0),
|
||||
ElementWiseOperation::kPOW);
|
||||
auto f_mean4 = network->addReduce(*f_pow4->getOutput(0), ReduceOperation::kAVG, 0XF, true);
|
||||
IElementWiseLayer *f_div4 = network->addElementWise(*backbone->getOutput(0), *f_mean4->getOutput(0),
|
||||
ElementWiseOperation::kDIV);
|
||||
|
||||
|
||||
ITensor *inputTensors[] = {f_div->getOutput(0), f_div2->getOutput(0), f_div3->getOutput(0), f_div4->getOutput(0)};
|
||||
|
||||
auto f_divdims = f_div->getOutput(0)->getDimensions();
|
||||
auto f_div2dims = f_div2->getOutput(0)->getDimensions();
|
||||
auto f_div3dims = f_div3->getOutput(0)->getDimensions();
|
||||
auto backbonedims = backbone->getOutput(0)->getDimensions();
|
||||
auto cat = network->addConcatenation(inputTensors, 4);
|
||||
Dims pcat = cat->getOutput(0)->getDimensions();
|
||||
IConvolutionLayer *container = network->addConvolutionNd(*cat->getOutput(0), 68, DimsHW{1, 1},
|
||||
weightMap["container.0.weight"],
|
||||
weightMap["container.0.bias"]);
|
||||
|
||||
auto logits = network->addReduce(*container->getOutput(0), ReduceOperation::kAVG, 0X04, false);
|
||||
|
||||
Dims dims = logits->getOutput(0)->getDimensions();
|
||||
std::cout << "logits shape " << dims.d[0] << " " << dims.d[1] << " " << dims.d[2] << std::endl;
|
||||
|
||||
logits->getOutput(0)->setName(OUTPUT_BLOB_NAME);
|
||||
network->markOutput(*logits->getOutput(0));
|
||||
|
||||
// Build engine
|
||||
builder->setMaxBatchSize(maxBatchSize);
|
||||
config->setMaxWorkspaceSize(16 * (1 << 20)); // 16MB
|
||||
#ifdef USE_FP16
|
||||
config->setFlag(BuilderFlag::kFP16);
|
||||
#endif
|
||||
std::cout << "Building engine, please wait for a while..." << std::endl;
|
||||
ICudaEngine *engine = builder->buildEngineWithConfig(*network, *config);
|
||||
std::cout << "Build engine successfully!" << std::endl;
|
||||
|
||||
// Don't need the network any more
|
||||
network->destroy();
|
||||
|
||||
// Release host memory
|
||||
for (auto &mem : weightMap) {
|
||||
free((void *) (mem.second.values));
|
||||
}
|
||||
|
||||
return engine;
|
||||
}
|
||||
|
||||
|
||||
void APIToModel(unsigned int maxBatchSize, IHostMemory **modelStream) {
|
||||
// Create builder
|
||||
IBuilder *builder = createInferBuilder(gLogger);
|
||||
IBuilderConfig *config = builder->createBuilderConfig();
|
||||
|
||||
// Create model to populate the network, then set the outputs and create an engine
|
||||
ICudaEngine *engine = createEngine(maxBatchSize, builder, config, DataType::kFLOAT);
|
||||
assert(engine != nullptr);
|
||||
|
||||
// Serialize the engine
|
||||
(*modelStream) = engine->serialize();
|
||||
|
||||
// Close everything down
|
||||
engine->destroy();
|
||||
builder->destroy();
|
||||
}
|
||||
|
||||
|
||||
void doInference(IExecutionContext &context, float *input, float *output, int batchSize) {
|
||||
const ICudaEngine &engine = context.getEngine();
|
||||
|
||||
// Pointers to input and output device buffers to pass to engine.
|
||||
// Engine requires exactly IEngine::getNbBindings() number of buffers.
|
||||
assert(engine.getNbBindings() == 2);
|
||||
void *buffers[2];
|
||||
|
||||
// In order to bind the buffers, we need to know the names of the input and output tensors.
|
||||
// Note that indices are guaranteed to be less than IEngine::getNbBindings()
|
||||
const int inputIndex = engine.getBindingIndex(INPUT_BLOB_NAME);
|
||||
const int outputIndex = engine.getBindingIndex(OUTPUT_BLOB_NAME);
|
||||
|
||||
// Create GPU buffers on device
|
||||
CHECK(cudaMalloc(&buffers[inputIndex], batchSize * 3 * INPUT_H * INPUT_W * sizeof(float)));
|
||||
CHECK(cudaMalloc(&buffers[outputIndex], batchSize * OUTPUT_SIZE * sizeof(float)));
|
||||
|
||||
// Create stream
|
||||
cudaStream_t stream;
|
||||
CHECK(cudaStreamCreate(&stream));
|
||||
|
||||
// DMA input batch data to device, infer on the batch asynchronously, and DMA output back to host
|
||||
CHECK(cudaMemcpyAsync(buffers[inputIndex], input, batchSize * 3 * INPUT_H * INPUT_W * sizeof(float),
|
||||
cudaMemcpyHostToDevice, stream));
|
||||
context.enqueue(batchSize, buffers, stream, nullptr);
|
||||
CHECK(cudaMemcpyAsync(output, buffers[outputIndex], batchSize * OUTPUT_SIZE * sizeof(float), cudaMemcpyDeviceToHost,
|
||||
stream));
|
||||
cudaStreamSynchronize(stream);
|
||||
|
||||
// Release stream and buffers
|
||||
cudaStreamDestroy(stream);
|
||||
CHECK(cudaFree(buffers[inputIndex]));
|
||||
CHECK(cudaFree(buffers[outputIndex]));
|
||||
}
|
||||
|
||||
int main(int argc, char **argv) {
|
||||
cudaSetDevice(DEVICE);
|
||||
// create a model using the API directly and serialize it to a stream
|
||||
char *trtModelStream{nullptr};
|
||||
size_t size{0};
|
||||
if (argc == 2 && std::string(argv[1]) == "-s") {
|
||||
IHostMemory *modelStream{nullptr};
|
||||
APIToModel(BATCH_SIZE, &modelStream);
|
||||
assert(modelStream != nullptr);
|
||||
std::ofstream p("LPRnet.engine", std::ios::binary);
|
||||
if (!p) {
|
||||
std::cerr << "could not open plan output file" << std::endl;
|
||||
return -1;
|
||||
}
|
||||
p.write(reinterpret_cast<const char *>(modelStream->data()), modelStream->size());
|
||||
modelStream->destroy();
|
||||
return 0;
|
||||
} else if (argc == 2 && std::string(argv[1]) == "-d") {
|
||||
std::ifstream file("LPRnet.engine", std::ios::binary);
|
||||
if (file.good()) {
|
||||
file.seekg(0, file.end);
|
||||
size = file.tellg();
|
||||
file.seekg(0, file.beg);
|
||||
trtModelStream = new char[size];
|
||||
assert(trtModelStream);
|
||||
file.read(trtModelStream, size);
|
||||
file.close();
|
||||
}
|
||||
} else {
|
||||
std::cerr << "arguments not right!" << std::endl;
|
||||
std::cerr << "./LPRnet -s // serialize model to plan file" << std::endl;
|
||||
std::cerr << "./LPRnet -d ../samples // deserialize plan file and run inference" << std::endl;
|
||||
return -1;
|
||||
}
|
||||
|
||||
// prepare input data ---------------------------
|
||||
static float data[BATCH_SIZE * 3 * INPUT_H * INPUT_W];
|
||||
|
||||
cv::Mat img = cv::imread("../1.jpg");
|
||||
cv::Mat pr_img;
|
||||
cv::resize(img, pr_img, cv::Size(INPUT_W, INPUT_H), 0, 0, cv::INTER_CUBIC);
|
||||
// For multi-batch, I feed the same image multiple times.
|
||||
// If you want to process different images in a batch, you need adapt it.
|
||||
cv::Mat blob = cv::dnn::blobFromImage(pr_img, 0.0078125, pr_img.size(), cv::Scalar(127.5, 127.5, 127.5), true,
|
||||
false);
|
||||
|
||||
IRuntime *runtime = createInferRuntime(gLogger);
|
||||
assert(runtime != nullptr);
|
||||
ICudaEngine *engine = runtime->deserializeCudaEngine(trtModelStream, size);
|
||||
//ICudaEngine* engine = runtime->deserializeCudaEngine(trtModelStream, size, nullptr);
|
||||
assert(engine != nullptr);
|
||||
IExecutionContext *context = engine->createExecutionContext();
|
||||
assert(context != nullptr);
|
||||
|
||||
// Run inference
|
||||
static float prob[BATCH_SIZE * OUTPUT_SIZE];
|
||||
auto start = std::chrono::system_clock::now();
|
||||
doInference(*context, blob.ptr<float>(0), prob, BATCH_SIZE);
|
||||
auto end = std::chrono::system_clock::now();
|
||||
std::cout << std::chrono::duration_cast<std::chrono::microseconds>(end - start).count() << "us" << std::endl;
|
||||
std::vector<int> preds;
|
||||
std::cout << std::endl;
|
||||
for (int i = 0; i < 18; i++) {
|
||||
int maxj = 0;
|
||||
for (int j = 0; j < 68; j++) {
|
||||
if (prob[i + 18 * j] > prob[i + 18 * maxj]) maxj = j;
|
||||
}
|
||||
preds.push_back(maxj);
|
||||
}
|
||||
int pre_c = preds[0];
|
||||
std::vector<int> no_repeat_blank_label;
|
||||
for (auto c: preds) {
|
||||
if (c == pre_c || c == 68 - 1) {
|
||||
if (c == 68 - 1) pre_c = c;
|
||||
continue;
|
||||
}
|
||||
no_repeat_blank_label.push_back(c);
|
||||
pre_c = c;
|
||||
}
|
||||
std::string str;
|
||||
for (auto v: no_repeat_blank_label) {
|
||||
str += alphabet[v];
|
||||
}
|
||||
std::cout<<"result:"<<str<<std::endl;
|
||||
// Destroy the engine
|
||||
context->destroy();
|
||||
engine->destroy();
|
||||
runtime->destroy();
|
||||
|
||||
|
||||
return 0;
|
||||
}
|
||||
28
lprnet/README.md
Normal file
28
lprnet/README.md
Normal file
@ -0,0 +1,28 @@
|
||||
#
|
||||
|
||||
The Pytorch implementation is [xuexingyu24/License_Plate_Detection_Pytorch](https://github.com/xuexingyu24/License_Plate_Detection_Pytorch).
|
||||
|
||||
## How to Run
|
||||
|
||||
```
|
||||
1. generate LPRnet.wts from pytorch
|
||||
|
||||
git clone https://github.com/wang-xinyu/tensorrtx.git
|
||||
git clone https://github.com/xuexingyu24/License_Plate_Detection_Pytorch.git
|
||||
|
||||
// copy tensorrtx/LRPnet/gen_wts.py License_Plate_Detection_Pytorch
|
||||
// go to License_Plate_Detection_Pytorch/
|
||||
python genwts.py
|
||||
// a file 'LPRnet.wts' will be generated.
|
||||
|
||||
2. build LPRnet and run
|
||||
|
||||
// put LPRnet.wts into tensorrtx/LPRnet
|
||||
// go to tensorrtx/LPRnet
|
||||
mkdir build
|
||||
cd build
|
||||
cmake ..
|
||||
make
|
||||
sudo ./LPRnet -s // serialize model to plan file i.e. 'LPRnet.engine'
|
||||
do inference
|
||||
sudo ./LPRnet -d // deserialize plan file and run inference
|
||||
42
lprnet/genwts.py
Normal file
42
lprnet/genwts.py
Normal file
@ -0,0 +1,42 @@
|
||||
import torch
|
||||
from torch.autograd import Variable
|
||||
|
||||
from LPRNet.model import LPRNET
|
||||
import struct
|
||||
|
||||
model_path = './weights/Final_LPRNet_model.pth'
|
||||
CHARS = ['京', '沪', '津', '渝', '冀', '晋', '蒙', '辽', '吉', '黑',
|
||||
'苏', '浙', '皖', '闽', '赣', '鲁', '豫', '鄂', '湘', '粤',
|
||||
'桂', '琼', '川', '贵', '云', '藏', '陕', '甘', '青', '宁',
|
||||
'新',
|
||||
'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
|
||||
'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'J', 'K',
|
||||
'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'U', 'V',
|
||||
'W', 'X', 'Y', 'Z', 'I', 'O', '-'
|
||||
]
|
||||
model = LPRNET.LPRNet(class_num=len(CHARS), dropout_rate=0)
|
||||
if torch.cuda.is_available():
|
||||
model = model.cuda()
|
||||
print('loading pretrained model from %s' % model_path)
|
||||
model.load_state_dict(torch.load(model_path))
|
||||
|
||||
image = torch.ones(1, 3, 24, 94)
|
||||
if torch.cuda.is_available():
|
||||
image = image.cuda()
|
||||
|
||||
model.eval()
|
||||
print(model)
|
||||
print('image shape ', image.shape)
|
||||
preds = model(image)
|
||||
|
||||
f = open("LPRNet.wts", 'w')
|
||||
f.write("{}\n".format(len(model.state_dict().keys())))
|
||||
for k, v in model.state_dict().items():
|
||||
print('key: ', k)
|
||||
print('value: ', v.shape)
|
||||
vr = v.reshape(-1).cpu().numpy()
|
||||
f.write("{} {}".format(k, len(vr)))
|
||||
for vv in vr:
|
||||
f.write(" ")
|
||||
f.write(struct.pack(">f", float(vv)).hex())
|
||||
f.write("\n")
|
||||
503
lprnet/logging.h
Normal file
503
lprnet/logging.h
Normal file
@ -0,0 +1,503 @@
|
||||
/*
|
||||
* Copyright (c) 2019, NVIDIA CORPORATION. All rights reserved.
|
||||
*
|
||||
* Licensed under the Apache License, Version 2.0 (the "License");
|
||||
* you may not use this file except in compliance with the License.
|
||||
* You may obtain a copy of the License at
|
||||
*
|
||||
* http://www.apache.org/licenses/LICENSE-2.0
|
||||
*
|
||||
* Unless required by applicable law or agreed to in writing, software
|
||||
* distributed under the License is distributed on an "AS IS" BASIS,
|
||||
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
|
||||
* See the License for the specific language governing permissions and
|
||||
* limitations under the License.
|
||||
*/
|
||||
|
||||
#ifndef TENSORRT_LOGGING_H
|
||||
#define TENSORRT_LOGGING_H
|
||||
|
||||
#include "NvInferRuntimeCommon.h"
|
||||
#include <cassert>
|
||||
#include <ctime>
|
||||
#include <iomanip>
|
||||
#include <iostream>
|
||||
#include <ostream>
|
||||
#include <sstream>
|
||||
#include <string>
|
||||
|
||||
using Severity = nvinfer1::ILogger::Severity;
|
||||
|
||||
class LogStreamConsumerBuffer : public std::stringbuf
|
||||
{
|
||||
public:
|
||||
LogStreamConsumerBuffer(std::ostream& stream, const std::string& prefix, bool shouldLog)
|
||||
: mOutput(stream)
|
||||
, mPrefix(prefix)
|
||||
, mShouldLog(shouldLog)
|
||||
{
|
||||
}
|
||||
|
||||
LogStreamConsumerBuffer(LogStreamConsumerBuffer&& other)
|
||||
: mOutput(other.mOutput)
|
||||
{
|
||||
}
|
||||
|
||||
~LogStreamConsumerBuffer()
|
||||
{
|
||||
// std::streambuf::pbase() gives a pointer to the beginning of the buffered part of the output sequence
|
||||
// std::streambuf::pptr() gives a pointer to the current position of the output sequence
|
||||
// if the pointer to the beginning is not equal to the pointer to the current position,
|
||||
// call putOutput() to log the output to the stream
|
||||
if (pbase() != pptr())
|
||||
{
|
||||
putOutput();
|
||||
}
|
||||
}
|
||||
|
||||
// synchronizes the stream buffer and returns 0 on success
|
||||
// synchronizing the stream buffer consists of inserting the buffer contents into the stream,
|
||||
// resetting the buffer and flushing the stream
|
||||
virtual int sync()
|
||||
{
|
||||
putOutput();
|
||||
return 0;
|
||||
}
|
||||
|
||||
void putOutput()
|
||||
{
|
||||
if (mShouldLog)
|
||||
{
|
||||
// prepend timestamp
|
||||
std::time_t timestamp = std::time(nullptr);
|
||||
tm* tm_local = std::localtime(×tamp);
|
||||
std::cout << "[";
|
||||
std::cout << std::setw(2) << std::setfill('0') << 1 + tm_local->tm_mon << "/";
|
||||
std::cout << std::setw(2) << std::setfill('0') << tm_local->tm_mday << "/";
|
||||
std::cout << std::setw(4) << std::setfill('0') << 1900 + tm_local->tm_year << "-";
|
||||
std::cout << std::setw(2) << std::setfill('0') << tm_local->tm_hour << ":";
|
||||
std::cout << std::setw(2) << std::setfill('0') << tm_local->tm_min << ":";
|
||||
std::cout << std::setw(2) << std::setfill('0') << tm_local->tm_sec << "] ";
|
||||
// std::stringbuf::str() gets the string contents of the buffer
|
||||
// insert the buffer contents pre-appended by the appropriate prefix into the stream
|
||||
mOutput << mPrefix << str();
|
||||
// set the buffer to empty
|
||||
str("");
|
||||
// flush the stream
|
||||
mOutput.flush();
|
||||
}
|
||||
}
|
||||
|
||||
void setShouldLog(bool shouldLog)
|
||||
{
|
||||
mShouldLog = shouldLog;
|
||||
}
|
||||
|
||||
private:
|
||||
std::ostream& mOutput;
|
||||
std::string mPrefix;
|
||||
bool mShouldLog;
|
||||
};
|
||||
|
||||
//!
|
||||
//! \class LogStreamConsumerBase
|
||||
//! \brief Convenience object used to initialize LogStreamConsumerBuffer before std::ostream in LogStreamConsumer
|
||||
//!
|
||||
class LogStreamConsumerBase
|
||||
{
|
||||
public:
|
||||
LogStreamConsumerBase(std::ostream& stream, const std::string& prefix, bool shouldLog)
|
||||
: mBuffer(stream, prefix, shouldLog)
|
||||
{
|
||||
}
|
||||
|
||||
protected:
|
||||
LogStreamConsumerBuffer mBuffer;
|
||||
};
|
||||
|
||||
//!
|
||||
//! \class LogStreamConsumer
|
||||
//! \brief Convenience object used to facilitate use of C++ stream syntax when logging messages.
|
||||
//! Order of base classes is LogStreamConsumerBase and then std::ostream.
|
||||
//! This is because the LogStreamConsumerBase class is used to initialize the LogStreamConsumerBuffer member field
|
||||
//! in LogStreamConsumer and then the address of the buffer is passed to std::ostream.
|
||||
//! This is necessary to prevent the address of an uninitialized buffer from being passed to std::ostream.
|
||||
//! Please do not change the order of the parent classes.
|
||||
//!
|
||||
class LogStreamConsumer : protected LogStreamConsumerBase, public std::ostream
|
||||
{
|
||||
public:
|
||||
//! \brief Creates a LogStreamConsumer which logs messages with level severity.
|
||||
//! Reportable severity determines if the messages are severe enough to be logged.
|
||||
LogStreamConsumer(Severity reportableSeverity, Severity severity)
|
||||
: LogStreamConsumerBase(severityOstream(severity), severityPrefix(severity), severity <= reportableSeverity)
|
||||
, std::ostream(&mBuffer) // links the stream buffer with the stream
|
||||
, mShouldLog(severity <= reportableSeverity)
|
||||
, mSeverity(severity)
|
||||
{
|
||||
}
|
||||
|
||||
LogStreamConsumer(LogStreamConsumer&& other)
|
||||
: LogStreamConsumerBase(severityOstream(other.mSeverity), severityPrefix(other.mSeverity), other.mShouldLog)
|
||||
, std::ostream(&mBuffer) // links the stream buffer with the stream
|
||||
, mShouldLog(other.mShouldLog)
|
||||
, mSeverity(other.mSeverity)
|
||||
{
|
||||
}
|
||||
|
||||
void setReportableSeverity(Severity reportableSeverity)
|
||||
{
|
||||
mShouldLog = mSeverity <= reportableSeverity;
|
||||
mBuffer.setShouldLog(mShouldLog);
|
||||
}
|
||||
|
||||
private:
|
||||
static std::ostream& severityOstream(Severity severity)
|
||||
{
|
||||
return severity >= Severity::kINFO ? std::cout : std::cerr;
|
||||
}
|
||||
|
||||
static std::string severityPrefix(Severity severity)
|
||||
{
|
||||
switch (severity)
|
||||
{
|
||||
case Severity::kINTERNAL_ERROR: return "[F] ";
|
||||
case Severity::kERROR: return "[E] ";
|
||||
case Severity::kWARNING: return "[W] ";
|
||||
case Severity::kINFO: return "[I] ";
|
||||
case Severity::kVERBOSE: return "[V] ";
|
||||
default: assert(0); return "";
|
||||
}
|
||||
}
|
||||
|
||||
bool mShouldLog;
|
||||
Severity mSeverity;
|
||||
};
|
||||
|
||||
//! \class Logger
|
||||
//!
|
||||
//! \brief Class which manages logging of TensorRT tools and samples
|
||||
//!
|
||||
//! \details This class provides a common interface for TensorRT tools and samples to log information to the console,
|
||||
//! and supports logging two types of messages:
|
||||
//!
|
||||
//! - Debugging messages with an associated severity (info, warning, error, or internal error/fatal)
|
||||
//! - Test pass/fail messages
|
||||
//!
|
||||
//! The advantage of having all samples use this class for logging as opposed to emitting directly to stdout/stderr is
|
||||
//! that the logic for controlling the verbosity and formatting of sample output is centralized in one location.
|
||||
//!
|
||||
//! In the future, this class could be extended to support dumping test results to a file in some standard format
|
||||
//! (for example, JUnit XML), and providing additional metadata (e.g. timing the duration of a test run).
|
||||
//!
|
||||
//! TODO: For backwards compatibility with existing samples, this class inherits directly from the nvinfer1::ILogger
|
||||
//! interface, which is problematic since there isn't a clean separation between messages coming from the TensorRT
|
||||
//! library and messages coming from the sample.
|
||||
//!
|
||||
//! In the future (once all samples are updated to use Logger::getTRTLogger() to access the ILogger) we can refactor the
|
||||
//! class to eliminate the inheritance and instead make the nvinfer1::ILogger implementation a member of the Logger
|
||||
//! object.
|
||||
|
||||
class Logger : public nvinfer1::ILogger
|
||||
{
|
||||
public:
|
||||
Logger(Severity severity = Severity::kWARNING)
|
||||
: mReportableSeverity(severity)
|
||||
{
|
||||
}
|
||||
|
||||
//!
|
||||
//! \enum TestResult
|
||||
//! \brief Represents the state of a given test
|
||||
//!
|
||||
enum class TestResult
|
||||
{
|
||||
kRUNNING, //!< The test is running
|
||||
kPASSED, //!< The test passed
|
||||
kFAILED, //!< The test failed
|
||||
kWAIVED //!< The test was waived
|
||||
};
|
||||
|
||||
//!
|
||||
//! \brief Forward-compatible method for retrieving the nvinfer::ILogger associated with this Logger
|
||||
//! \return The nvinfer1::ILogger associated with this Logger
|
||||
//!
|
||||
//! TODO Once all samples are updated to use this method to register the logger with TensorRT,
|
||||
//! we can eliminate the inheritance of Logger from ILogger
|
||||
//!
|
||||
nvinfer1::ILogger& getTRTLogger()
|
||||
{
|
||||
return *this;
|
||||
}
|
||||
|
||||
//!
|
||||
//! \brief Implementation of the nvinfer1::ILogger::log() virtual method
|
||||
//!
|
||||
//! Note samples should not be calling this function directly; it will eventually go away once we eliminate the
|
||||
//! inheritance from nvinfer1::ILogger
|
||||
//!
|
||||
void log(Severity severity, const char* msg) override
|
||||
{
|
||||
LogStreamConsumer(mReportableSeverity, severity) << "[TRT] " << std::string(msg) << std::endl;
|
||||
}
|
||||
|
||||
//!
|
||||
//! \brief Method for controlling the verbosity of logging output
|
||||
//!
|
||||
//! \param severity The logger will only emit messages that have severity of this level or higher.
|
||||
//!
|
||||
void setReportableSeverity(Severity severity)
|
||||
{
|
||||
mReportableSeverity = severity;
|
||||
}
|
||||
|
||||
//!
|
||||
//! \brief Opaque handle that holds logging information for a particular test
|
||||
//!
|
||||
//! This object is an opaque handle to information used by the Logger to print test results.
|
||||
//! The sample must call Logger::defineTest() in order to obtain a TestAtom that can be used
|
||||
//! with Logger::reportTest{Start,End}().
|
||||
//!
|
||||
class TestAtom
|
||||
{
|
||||
public:
|
||||
TestAtom(TestAtom&&) = default;
|
||||
|
||||
private:
|
||||
friend class Logger;
|
||||
|
||||
TestAtom(bool started, const std::string& name, const std::string& cmdline)
|
||||
: mStarted(started)
|
||||
, mName(name)
|
||||
, mCmdline(cmdline)
|
||||
{
|
||||
}
|
||||
|
||||
bool mStarted;
|
||||
std::string mName;
|
||||
std::string mCmdline;
|
||||
};
|
||||
|
||||
//!
|
||||
//! \brief Define a test for logging
|
||||
//!
|
||||
//! \param[in] name The name of the test. This should be a string starting with
|
||||
//! "TensorRT" and containing dot-separated strings containing
|
||||
//! the characters [A-Za-z0-9_].
|
||||
//! For example, "TensorRT.sample_googlenet"
|
||||
//! \param[in] cmdline The command line used to reproduce the test
|
||||
//
|
||||
//! \return a TestAtom that can be used in Logger::reportTest{Start,End}().
|
||||
//!
|
||||
static TestAtom defineTest(const std::string& name, const std::string& cmdline)
|
||||
{
|
||||
return TestAtom(false, name, cmdline);
|
||||
}
|
||||
|
||||
//!
|
||||
//! \brief A convenience overloaded version of defineTest() that accepts an array of command-line arguments
|
||||
//! as input
|
||||
//!
|
||||
//! \param[in] name The name of the test
|
||||
//! \param[in] argc The number of command-line arguments
|
||||
//! \param[in] argv The array of command-line arguments (given as C strings)
|
||||
//!
|
||||
//! \return a TestAtom that can be used in Logger::reportTest{Start,End}().
|
||||
static TestAtom defineTest(const std::string& name, int argc, char const* const* argv)
|
||||
{
|
||||
auto cmdline = genCmdlineString(argc, argv);
|
||||
return defineTest(name, cmdline);
|
||||
}
|
||||
|
||||
//!
|
||||
//! \brief Report that a test has started.
|
||||
//!
|
||||
//! \pre reportTestStart() has not been called yet for the given testAtom
|
||||
//!
|
||||
//! \param[in] testAtom The handle to the test that has started
|
||||
//!
|
||||
static void reportTestStart(TestAtom& testAtom)
|
||||
{
|
||||
reportTestResult(testAtom, TestResult::kRUNNING);
|
||||
assert(!testAtom.mStarted);
|
||||
testAtom.mStarted = true;
|
||||
}
|
||||
|
||||
//!
|
||||
//! \brief Report that a test has ended.
|
||||
//!
|
||||
//! \pre reportTestStart() has been called for the given testAtom
|
||||
//!
|
||||
//! \param[in] testAtom The handle to the test that has ended
|
||||
//! \param[in] result The result of the test. Should be one of TestResult::kPASSED,
|
||||
//! TestResult::kFAILED, TestResult::kWAIVED
|
||||
//!
|
||||
static void reportTestEnd(const TestAtom& testAtom, TestResult result)
|
||||
{
|
||||
assert(result != TestResult::kRUNNING);
|
||||
assert(testAtom.mStarted);
|
||||
reportTestResult(testAtom, result);
|
||||
}
|
||||
|
||||
static int reportPass(const TestAtom& testAtom)
|
||||
{
|
||||
reportTestEnd(testAtom, TestResult::kPASSED);
|
||||
return EXIT_SUCCESS;
|
||||
}
|
||||
|
||||
static int reportFail(const TestAtom& testAtom)
|
||||
{
|
||||
reportTestEnd(testAtom, TestResult::kFAILED);
|
||||
return EXIT_FAILURE;
|
||||
}
|
||||
|
||||
static int reportWaive(const TestAtom& testAtom)
|
||||
{
|
||||
reportTestEnd(testAtom, TestResult::kWAIVED);
|
||||
return EXIT_SUCCESS;
|
||||
}
|
||||
|
||||
static int reportTest(const TestAtom& testAtom, bool pass)
|
||||
{
|
||||
return pass ? reportPass(testAtom) : reportFail(testAtom);
|
||||
}
|
||||
|
||||
Severity getReportableSeverity() const
|
||||
{
|
||||
return mReportableSeverity;
|
||||
}
|
||||
|
||||
private:
|
||||
//!
|
||||
//! \brief returns an appropriate string for prefixing a log message with the given severity
|
||||
//!
|
||||
static const char* severityPrefix(Severity severity)
|
||||
{
|
||||
switch (severity)
|
||||
{
|
||||
case Severity::kINTERNAL_ERROR: return "[F] ";
|
||||
case Severity::kERROR: return "[E] ";
|
||||
case Severity::kWARNING: return "[W] ";
|
||||
case Severity::kINFO: return "[I] ";
|
||||
case Severity::kVERBOSE: return "[V] ";
|
||||
default: assert(0); return "";
|
||||
}
|
||||
}
|
||||
|
||||
//!
|
||||
//! \brief returns an appropriate string for prefixing a test result message with the given result
|
||||
//!
|
||||
static const char* testResultString(TestResult result)
|
||||
{
|
||||
switch (result)
|
||||
{
|
||||
case TestResult::kRUNNING: return "RUNNING";
|
||||
case TestResult::kPASSED: return "PASSED";
|
||||
case TestResult::kFAILED: return "FAILED";
|
||||
case TestResult::kWAIVED: return "WAIVED";
|
||||
default: assert(0); return "";
|
||||
}
|
||||
}
|
||||
|
||||
//!
|
||||
//! \brief returns an appropriate output stream (cout or cerr) to use with the given severity
|
||||
//!
|
||||
static std::ostream& severityOstream(Severity severity)
|
||||
{
|
||||
return severity >= Severity::kINFO ? std::cout : std::cerr;
|
||||
}
|
||||
|
||||
//!
|
||||
//! \brief method that implements logging test results
|
||||
//!
|
||||
static void reportTestResult(const TestAtom& testAtom, TestResult result)
|
||||
{
|
||||
severityOstream(Severity::kINFO) << "&&&& " << testResultString(result) << " " << testAtom.mName << " # "
|
||||
<< testAtom.mCmdline << std::endl;
|
||||
}
|
||||
|
||||
//!
|
||||
//! \brief generate a command line string from the given (argc, argv) values
|
||||
//!
|
||||
static std::string genCmdlineString(int argc, char const* const* argv)
|
||||
{
|
||||
std::stringstream ss;
|
||||
for (int i = 0; i < argc; i++)
|
||||
{
|
||||
if (i > 0)
|
||||
ss << " ";
|
||||
ss << argv[i];
|
||||
}
|
||||
return ss.str();
|
||||
}
|
||||
|
||||
Severity mReportableSeverity;
|
||||
};
|
||||
|
||||
namespace
|
||||
{
|
||||
|
||||
//!
|
||||
//! \brief produces a LogStreamConsumer object that can be used to log messages of severity kVERBOSE
|
||||
//!
|
||||
//! Example usage:
|
||||
//!
|
||||
//! LOG_VERBOSE(logger) << "hello world" << std::endl;
|
||||
//!
|
||||
inline LogStreamConsumer LOG_VERBOSE(const Logger& logger)
|
||||
{
|
||||
return LogStreamConsumer(logger.getReportableSeverity(), Severity::kVERBOSE);
|
||||
}
|
||||
|
||||
//!
|
||||
//! \brief produces a LogStreamConsumer object that can be used to log messages of severity kINFO
|
||||
//!
|
||||
//! Example usage:
|
||||
//!
|
||||
//! LOG_INFO(logger) << "hello world" << std::endl;
|
||||
//!
|
||||
inline LogStreamConsumer LOG_INFO(const Logger& logger)
|
||||
{
|
||||
return LogStreamConsumer(logger.getReportableSeverity(), Severity::kINFO);
|
||||
}
|
||||
|
||||
//!
|
||||
//! \brief produces a LogStreamConsumer object that can be used to log messages of severity kWARNING
|
||||
//!
|
||||
//! Example usage:
|
||||
//!
|
||||
//! LOG_WARN(logger) << "hello world" << std::endl;
|
||||
//!
|
||||
inline LogStreamConsumer LOG_WARN(const Logger& logger)
|
||||
{
|
||||
return LogStreamConsumer(logger.getReportableSeverity(), Severity::kWARNING);
|
||||
}
|
||||
|
||||
//!
|
||||
//! \brief produces a LogStreamConsumer object that can be used to log messages of severity kERROR
|
||||
//!
|
||||
//! Example usage:
|
||||
//!
|
||||
//! LOG_ERROR(logger) << "hello world" << std::endl;
|
||||
//!
|
||||
inline LogStreamConsumer LOG_ERROR(const Logger& logger)
|
||||
{
|
||||
return LogStreamConsumer(logger.getReportableSeverity(), Severity::kERROR);
|
||||
}
|
||||
|
||||
//!
|
||||
//! \brief produces a LogStreamConsumer object that can be used to log messages of severity kINTERNAL_ERROR
|
||||
// ("fatal" severity)
|
||||
//!
|
||||
//! Example usage:
|
||||
//!
|
||||
//! LOG_FATAL(logger) << "hello world" << std::endl;
|
||||
//!
|
||||
inline LogStreamConsumer LOG_FATAL(const Logger& logger)
|
||||
{
|
||||
return LogStreamConsumer(logger.getReportableSeverity(), Severity::kINTERNAL_ERROR);
|
||||
}
|
||||
|
||||
} // anonymous namespace
|
||||
|
||||
#endif // TENSORRT_LOGGING_H
|
||||
Loading…
Reference in New Issue
Block a user